@@ -65,7 +65,12 @@ base_client_new(Wm *w, Window win)
    /* Basic attributes */
 
    XGetWindowAttributes(w->dpy, win, &attr);
-   
+
+   /*
+    * What todo about attr.class == InputOnly case ?
+    * Should never happen and likely throw X error to untrap.
+    */
+
    c->x      = attr.x;
    c->y      = attr.y;
    c->cmap   = attr.colormap;
@@ -279,6 +284,7 @@ base_client_new(Wm *w, Window win)
 
    if (misc_untrap_xerrors()) 	/* An X error occured */
      {				/* Likely client died */
+       dbg("%s() looks like client just died on us\n", __func__);
        c->frame = None;
        base_client_destroy(c);
        return NULL;
@@ -362,8 +368,12 @@ base_client_process_name(Client *c)
       
       if (max)
 	{
-	  tmp_name = alloca(sizeof(char) * (strlen(c->name) + 7));
-	  sprintf(tmp_name, "%s <%i>", c->name, ++max);
+	  int tmp_name_sz;
+	  
+	  tmp_name_sz = strlen(c->name) + 7;;
+	  tmp_name    = alloca(tmp_name_sz);
+	  snprintf(tmp_name, tmp_name_sz, "%s <%i>", c->name, ++max);
+
 	  free(c->name);
 
 	  XStoreName(w->dpy, c->window, tmp_name);
@@ -495,6 +505,11 @@ base_client_destroy(Client *c)
 #endif
      }
 
+   /* Whatever we do the below is very likely to fire off a ( harmless ) 
+    *  X Error or two. Therefore we trap, just to quiten the warnings.
+   */
+   misc_trap_xerrors();
+
 #ifdef USE_LIBSN
    wm_sn_cycle_remove(w, c->window);
 #endif       
@@ -540,9 +555,13 @@ base_client_destroy(Client *c)
     if (c == w->focused_client)
       w->focused_client = NULL;
 
+    /* Be sure to flush out all calls before we untrap.
+     * Important here as the above does alot.
+    */
+    XSync(w->dpy, False);
+    misc_untrap_xerrors();
+
     free(c);
-
-
 }
 
 
