@@ -47,7 +47,9 @@ client_get_state(Client *c)
   int           real_format;
   unsigned long items_read, items_left;
   long         *data = NULL, state = WithdrawnState;
-  
+
+  misc_trap_xerrors(); 
+
   if (XGetWindowProperty(w->dpy, c->window,
 			 w->atoms[WM_STATE], 0L, 2L, False,
 			 w->atoms[WM_STATE], &real_type, &real_format,
@@ -56,6 +58,9 @@ client_get_state(Client *c)
       && items_read)
     state = *data;
 
+  if (misc_untrap_xerrors()) 	/* Just in case */
+    state = WithdrawnState; 
+
   if (data)
     XFree(data);
 
@@ -273,6 +278,8 @@ client_set_focus(Client *c)
       dbg("%s() called, setting focus to %s\n", 
 	  __func__, c->name);
 
+      ewmh_set_active(w);
+
       return True;
     }
   
@@ -504,8 +511,39 @@ client_init_backing_mask (Client *c,
   XFreeGC(w->dpy, shape_gc);
 }
 
+void
+client_button_init(Client         *c, 
+		   Window          win_parent, 
+		   MBClientButton *b,
+		   int             x, 
+		   int             y, 
+		   int             width, 
+		   int             height,
+		   Bool            want_inputonly,
+		   void           *data)
+{
+  Wm *w = c->wm;
 
-/* Decoration button stuff, could probably do with a good cleanup/rewrite */
+  int                  class = CopyFromParent;
+  XSetWindowAttributes attr;
+
+  attr.override_redirect = True; 
+  attr.event_mask        = ExposureMask;
+  
+  if (want_inputonly ) class = InputOnly;	      
+  
+  b->x = x; b->y = y; b->w = width; b->h = height; b->data = data;
+  
+  b->win = XCreateWindow(w->dpy, win_parent, 
+			 x, y, width, height, 0,
+			 CopyFromParent, 
+			 class, 
+			 CopyFromParent,
+			 CWOverrideRedirect|CWEventMask, 
+			 &attr);
+
+  XMapWindow(w->dpy, b->win);
+}
 
 MBClientButton*
 client_button_new(Client *c, 
@@ -517,25 +555,13 @@ client_button_new(Client *c,
 		  Bool    want_inputonly, 
 		  void   *data )
 {
-  Wm *w = c->wm;
-  XSetWindowAttributes attr;
-  int                  class = CopyFromParent;
   MBClientButton      *b = malloc(sizeof(MBClientButton));
 
   memset(b, 0, sizeof(MBClientButton));
-  
-  attr.override_redirect = True; 
-  attr.event_mask = ExposureMask;
-  
-  if (want_inputonly ) class = InputOnly;	      
-  
-  b->x = x; b->y = y; b->w = width; b->h = height; b->data = data;
-  
-  b->win = XCreateWindow(w->dpy, win_parent, x, y, width, height, 0,
-			 CopyFromParent, class, CopyFromParent,
-			 CWOverrideRedirect|CWEventMask, &attr);
 
-  XMapWindow(w->dpy, b->win);
+  client_button_init(c, win_parent, b, 
+		     x, y, width, height, 
+		     want_inputonly, data);
   return b;
 }
 
@@ -650,6 +676,7 @@ client_button_do_ops(Client       *c,
      if (b->press_activates)
        {
 	 XUngrabPointer(w->dpy, CurrentTime); 
+
 	 client_deliver_message(c, w->atoms[MB_GRAB_TRANSFER],
 				CurrentTime, e->subwindow, 0, 0, 0);
 	 return button_item->id;
