@@ -1,17 +1,22 @@
-/* matchbox - a lightweight window manager
-
-   Copyright 2002 Matthew Allum
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-*/
+/* 
+ *  Matchbox Window Manager - A lightweight window manager not for the
+ *                            desktop.
+ *
+ *  Authored By Matthew Allum <mallum@o-hand.com>
+ *
+ *  Copyright (c) 2002, 2004 OpenedHand Ltd - http://o-hand.com
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ */
 
 #ifndef _CLIENT_COMMON_H_
 #define _CLIENT_COMMON_H_
@@ -21,8 +26,11 @@
 #include "toolbar_client.h"
 #include "dockbar_client.h"
 #include "dialog_client.h"
+#include "list.h"
 #include "misc.h"
 
+#define client_title_frame(c) (c)->frames_decor[NORTH]
+
 void 
 client_set_state (Client *c, int state);
 
@@ -38,6 +46,7 @@ client_deliver_wm_protocol (Client *c, Atom delivery);
 void
 client_deliver_message(Client       *c, 
 		       Atom          delivery,
+		       unsigned long data0,
 		       unsigned long data1,
 		       unsigned long data2,
 		       unsigned long data3,
@@ -52,22 +61,35 @@ client_deliver_delete (Client *c);
 int 
 client_want_focus (Client *c);
 
+Bool
+client_set_focus(Client *c);
+
+void
+client_get_transient_list(Wm *w, MBList **list, Client *c);
+
+Client*
+client_get_highest_transient(Client *c, int client_flags);
+
 Client *
 client_get_next (Client* c, MBClientTypeEnum wanted);
 
 Client *
 client_get_prev (Client* c, MBClientTypeEnum wanted);
 
-void 
-client_init_backing (Client* c, int width, int height);
 
-/*
 void
-client_init_backing_mask (Client *c, 
-			  int     width, 
-			  int     height_north, 
-			  int     height_south );
-*/
+client_decor_frames_move_resize(Client *c, 
+				int     width_west, 
+				int     width_east, 
+				int     height_north,
+				int     height_south);
+
+void
+client_decor_frames_init(Client *c, 
+			 int     width_west, 
+			 int     width_east, 
+			 int     height_north,
+			 int     height_south);
 
 void
 client_init_backing_mask (Client *c, 
@@ -89,6 +111,17 @@ int
 client_button_do_ops (Client *c, XButtonEvent *e, int frame_type,
 		     int w, int h);
 
+void
+client_button_init(Client         *c, 
+		   Window          win_parent, 
+		   MBClientButton *b,
+		   int             x, 
+		   int             y, 
+		   int             width, 
+		   int             height,
+		   Bool            want_inputonly,
+		   void           *data);
+
 MBClientButton *
 client_button_new (Client *c, 
 		   Window  win_parent, 
