@@ -122,6 +122,7 @@ dialog_client_check_for_state_hints(Client *c)
 
   if (ewmh_state_check(c, c->wm->atoms[WINDOW_STATE_ABOVE]))
     c->flags |= CLIENT_HAS_ABOVE_STATE;
+
 }
 
 void
@@ -307,7 +308,8 @@ dialog_client_reparent(Client *c)
 {
   Wm *w = c->wm;
   XSetWindowAttributes attr;
-  
+  unsigned long        attr_mask;  
+
   int offset_north = dialog_client_title_height(c);
   int offset_south = 0, offset_west = 0, offset_east = 0;
   
@@ -317,9 +319,19 @@ dialog_client_reparent(Client *c)
   attr.background_pixel  = w->grey_col.pixel;
   attr.border_pixel      = 0;
   attr.event_mask        = ChildMask|ButtonPressMask|ExposureMask;
-  
-  attr.colormap          = c->cmap; /* Not used - needed ? */
-  
+
+  attr_mask              = CWOverrideRedirect|CWEventMask
+                           |CWBackPixel|CWBorderPixel ;
+
+#ifdef USE_COMPOSITE
+ /* Needed for argb wins, XXX need to figure this out .. */  
+  attr.colormap          = c->cmap;
+
+  if (c->cmap && !w->comp_engine_disabled) 
+    attr_mask = CWOverrideRedirect|CWEventMask
+                |CWBackPixel|CWBorderPixel|CWColormap ;
+#endif
+
   dbg("%s() want lowlight : wm:%i , client:%i\n", __func__,
       c->wm->config->dialog_shade, (c->flags & CLIENT_IS_MODAL_FLAG));
 #ifndef USE_COMPOSITE
@@ -350,8 +362,7 @@ dialog_client_reparent(Client *c)
 				     CopyFromParent, 
 				     CopyFromParent,
 #endif
-				     CWOverrideRedirect|CWEventMask
-				     |CWBackPixel|CWBorderPixel/*|CWColormap*/, 
+				     attr_mask,
 				     &attr);
      }
 
@@ -382,8 +393,13 @@ dialog_client_reparent(Client *c)
 					    CopyFromParent,
 					    CWOverrideRedirect|CWEventMask,
 					    &attr);
+
        XMapWindow(w->dpy, c->win_modal_blocker);
+
+       dbg("%s() created blocked win\n", __func__);
+       
        w->stack_n_items++;
+
      }
 
    XClearWindow(w->dpy, c->frame);
@@ -426,7 +442,7 @@ dialog_get_available_area(Client *c,
     }
   else
     {
-      Client *p = NULL;
+      Client *p = NULL, *main_client = wm_get_visible_main_client(w);
       Bool    have_toolbar = False;
 
      stack_enumerate(w, p)
@@ -436,17 +452,51 @@ dialog_get_available_area(Client *c,
 	  { have_toolbar = True; break; }
       }
 
-      *y      = wm_get_offsets_size(w, NORTH, NULL, True);
+      *y = wm_get_offsets_size(w, NORTH, NULL, True);
+
+      if (main_client && (main_client->flags & CLIENT_FULLSCREEN_FLAG))
+	{
+	  /* Fullscreen window present, allow dialogs to position themselves
+           * move or less anywhere
+           * 
+           * XXX: should we check is this dialog is trans for it or trans for
+           *      root ?
+	  */
+
+	  *height = w->dpy_height - *y;
+	  *x = 0;
+	  *width  = w->dpy_width;
+
+	  /* This mainly for alt toolbars so dialogs get positioned ok
+	   * for fullscreen.    
+           * Note the above wm_get_offsets calls with NORTH, not SOUTH,
+           * this is what below is basically doing. 
+	  */
+	  if (p != NULL && have_toolbar)
+	    *height -=  p->height;
+
+	  dbg("%s() height is %i, y is %i\n", __func__, *height, *y);
+
+	}
+      else
+	{
+	  /* if toolbar ( input window present ) dialogs can cover titlebars 
+	   * as can transient for root dialogs. 
+	   */
+	  if (!have_toolbar)
+	    *y  += main_client_title_height(c->trans);
+	  
+	  *height = w->dpy_height - *y - wm_get_offsets_size(w, SOUTH, 
+							     NULL, True);
+	  *x      = wm_get_offsets_size(w, WEST, NULL, True);
+	  *width  = w->dpy_width - *x - wm_get_offsets_size(w, EAST, 
+							    NULL, True);
+	}
+
+      dbg("%s() (toolbar) offsets south is %i\n", 
+	  __func__, wm_get_offsets_size(w, SOUTH, NULL, True));
 
-      /* if toolbar ( input window present ) dialogs can cover titlebars 
-       * as can transient for root dialogs. 
-       */
-      if (!have_toolbar)
-	*y  += main_client_title_height(c->trans);
 
-      *height = w->dpy_height - *y - wm_get_offsets_size(w, SOUTH, NULL, True);
-      *x      = wm_get_offsets_size(w, WEST, NULL, True);
-      *width  = w->dpy_width - *x - wm_get_offsets_size(w, EAST, NULL, True);
     }
 }
 
@@ -489,6 +539,9 @@ dialog_constrain_geometry(Client *c,
 
   dialog_get_available_area(c,&avail_x, &avail_y, &avail_width, &avail_height);
 
+  dbg("%s() getting available area (toolbar +%i+%i %ix%i)\n", 
+      __func__, avail_x, avail_y, avail_width, avail_height);
+
   /* Figure out window border offsets */
   dialog_client_get_offsets(c, &bdr_east, &bdr_south, &bdr_west);
   bdr_north = dialog_client_title_height(c);
@@ -566,11 +619,14 @@ dialog_init_geometry(Client *c)
     return;
 
   /* Allow decorationless dialogs to position themselves anywhere 
-   * But centered initially if 0,0 
+   * But centered initially if 0,0 ( splash screens ).
+   *
+   * However for 'menu' dialogs this doesn't apply as a fullscreen
+   * window may want a menu at 0,0, so we dont center them
   */
   if (c->flags & CLIENT_TITLE_HIDDEN_FLAG)
     {
-      if (c->x == 0 && c->y == 0)
+      if (!(c->flags & CLIENT_IS_MENU_DIALOG) && c->x == 0 && c->y == 0)
 	{
 	  if (c->height < w->dpy_height)
 	    c->y = (w->dpy_height - c->height)/2;
@@ -658,6 +714,11 @@ dialog_client_configure(Client *c)
 {
   dbg("%s() client has border only hint: %s\n",
       __func__, (c->flags & CLIENT_BORDERS_ONLY_FLAG) ? "yes" : "no");
+
+  dbg("%s() client has menu hint: %s\n",
+      __func__, (c->flags & CLIENT_IS_MENU_DIALOG) ? "yes" : "no");
+
+
   dialog_init_geometry(c);
 }
 
@@ -733,13 +794,16 @@ dialog_client_redraw(Client *c, Bool use_cache)
 			       total_w, offset_north);
 
       if (c->flags & CLIENT_ACCEPT_BUTTON_FLAG)
-	theme_frame_button_paint(c->wm->mbtheme, c, BUTTON_ACTION_ACCEPT, 
+	theme_frame_button_paint(w->mbtheme, c, BUTTON_ACTION_ACCEPT, 
 				 INACTIVE, FRAME_DIALOG,total_w, offset_north);
 
       if (c->flags & CLIENT_HELP_BUTTON_FLAG)
-	theme_frame_button_paint(c->wm->mbtheme, c, BUTTON_ACTION_HELP, 
+	theme_frame_button_paint(w->mbtheme, c, BUTTON_ACTION_HELP, 
 				 INACTIVE, FRAME_DIALOG,total_w, offset_north);
 
+      if (c->flags & CLIENT_CUSTOM_BUTTON_FLAG)
+	theme_frame_button_paint(w->mbtheme, c, BUTTON_ACTION_CUSTOM, 
+				 INACTIVE, FRAME_DIALOG, total_w,offset_north);
     }
 
 
@@ -758,6 +822,8 @@ dialog_client_redraw(Client *c, Bool use_cache)
 				0, 0, rects, 1, ShapeSet, 0 );
 
 #ifndef USE_COMPOSITE
+
+#if 0
       if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG)) 
 	{
 	  /* client->frame is our lowlighted window, so we only shape
@@ -783,6 +849,8 @@ dialog_client_redraw(Client *c, Bool use_cache)
 			     c->backing_masks[MSK_EAST], ShapeSet);
 	}
       else
+#endif
+
 #endif
 	{
 
@@ -845,13 +913,17 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 			       offset_north))
    {
       case BUTTON_ACTION_CLOSE:
-	 client_deliver_delete(c);
-	 break;
+	client_deliver_delete(c);
+	break;
       case BUTTON_ACTION_HELP:
-	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_HELP]);
-	 break;
+	client_deliver_wm_protocol(c, w->atoms[_NET_WM_CONTEXT_HELP]);
+	break;
       case BUTTON_ACTION_ACCEPT:
-	client_deliver_wm_protocol(c, c->wm->atoms[_NET_WM_CONTEXT_ACCEPT]);
+	client_deliver_wm_protocol(c, w->atoms[_NET_WM_CONTEXT_ACCEPT]);
+	break;
+      case BUTTON_ACTION_CUSTOM:
+	client_deliver_wm_protocol(c, w->atoms[_NET_WM_CONTEXT_CUSTOM]);
+	break;
       case -1: 		 /* Cancelled  */
 	 break;
       case 0:
@@ -903,9 +975,24 @@ dialog_client_button_press(Client *c, XButtonEvent *e)
 		  }
 		
 		XUngrabPointer(w->dpy, CurrentTime); 
+
+		/* For some odd reason the above make focus get 
+                 * lost ( for gtk apps at least ). We therefore
+                 * reset it.
+                 * ( we have to set focused client to NULL for set_focus()
+                 *   logic to work :( may a re_focus() is better ).
+		*/
+		if (w->focused_client == c)
+		  {
+		    w->focused_client = NULL;
+		    client_set_focus(c);
+		  }
+
 		XSync(w->dpy, False);
 	      }
 
+	    
+
 	    return;
 	  }
 
@@ -934,7 +1021,7 @@ dialog_client_drag(Client *c) /* drag box */
       != GrabSuccess)
     return;
 
- comp_engine_client_show(c->wm, c); 
+  comp_engine_client_show(c->wm, c); 
 
   c->flags |= CLIENT_IS_MOVING;
 
@@ -1010,6 +1097,8 @@ dialog_client_drag(Client *c) /* drag box */
 		      c->height + frm_size + offset_south);
 
 #ifndef USE_COMPOSITE	
+
+#if 0
 	if (c->wm->config->dialog_shade && (c->flags & CLIENT_IS_MODAL_FLAG))
 	  {
 	    XMoveResizeWindow(c->wm->dpy, 
@@ -1020,6 +1109,8 @@ dialog_client_drag(Client *c) /* drag box */
 			      c->height);
 	  } 
 	else
+#endif
+
 #endif /* USE_COMPOSITE */
 	  {
 	    XMoveWindow(c->wm->dpy, c->frame, c->x - offset_west,
@@ -1071,8 +1162,17 @@ dialog_client_set_focus_next(Client *c)
 {
   Wm *w = c->wm; 
 
+  /* We dont have focus, therefore dont suggest what to focus next */
+  if (w->focused_client != c)
+    return NULL;
+
   if (c->next_focused_client && c != c->next_focused_client)
-    client_set_focus(c->next_focused_client); 
+    {
+      if (w->focused_client == c)
+	w->focused_client = NULL;
+
+      client_set_focus(c->next_focused_client); 
+    }
   else
     {
       if (w->focused_client == c)
