@@ -113,7 +113,8 @@ ewmh_init(Wm *w)
     "_NET_WM_STATE_ABOVE",
     "WM_TRANSIENT_FOR",
     "_NET_WM_SYNC_REQUEST_COUNTER",
-    "_NET_WM_SYNC_REQUEST"
+    "_NET_WM_SYNC_REQUEST",
+    "_MB_CURRENT_APP_WINDOW"
   };
 
   XInternAtoms (w->dpy, atom_names, ATOM_COUNT,
@@ -154,7 +155,31 @@ ewmh_handle_root_message(Wm *w, XClientMessageEvent *e)
      {
        dbg("%s() got active window message for win %li", __func__, e->window);
        if ((c = wm_find_client(w, e->window, WINDOW)) != NULL)
-	 wm_activate_client(c);
+	 {
+	   if (c->type == MBCLIENT_TYPE_DIALOG
+	       && c->trans != NULL)
+	     {
+	       /* 
+                * If an attempt has been made to activate a hidden
+                * dialog, activate its parent app first.
+                *
+                * Note this is mainly to work with some task selectors
+                * ( eg the gnome one, which activates top dialog ).
+                *
+                * XXX wm_activate_client() should probably do this.
+	       */
+
+	       Client *parent = c->trans;
+
+	       while (parent->trans != NULL)
+		 parent = parent->trans;
+
+	       if (parent != wm_get_visible_main_client(w))
+		 wm_activate_client(parent);
+	     }
+
+	   wm_activate_client(c);
+	 }
        return 1;
      } 
    else if (e->message_type == w->atoms[_NET_CLOSE_WINDOW]) 
@@ -289,9 +314,9 @@ ewmh_update_lists(Wm *w)
 	  if (current_cycle->xid != None)
 	    {
 	      char str_tmp[256] = "";
-	      sprintf(str_tmp, "%s=%li|", 
-		      current_cycle->bin_name, 
-		      current_cycle->xid);
+	      snprintf(str_tmp, 256, "%s=%li|", 
+		       current_cycle->bin_name, 
+		       current_cycle->xid);
 	      
 	      strcat(bin_map_str, str_tmp);
 	      
@@ -382,8 +407,6 @@ ewmh_update_rects(Wm *w)
   val[3] = w->dpy_height - wm_get_offsets_size(w, NORTH, NULL, True)
     - wm_get_offsets_size(w, SOUTH, NULL, True);
 
-  dbg("%s(): val[3] is %li\n", __func__, val[3] );
-
   if (w->flags & DESKTOP_DECOR_FLAG)
     {
       /* Desktop is decorated, needs to know frame border sizes  */
@@ -395,13 +418,15 @@ ewmh_update_rects(Wm *w)
       val[3] -= theme_frame_defined_height_get(w->mbtheme, FRAME_MAIN); 
     }
 
-  dbg("%s(): val[3] now is %li\n", __func__, val[3] );
+  dbg("%s(): vals now is %li, %li, %li, %li ( root: %li )\n", 
+      __func__, val[0], val[1], val[2], val[3], w->root );
 
   XChangeProperty(w->dpy, w->root, w->atoms[_NET_WORKAREA],
 		  XA_CARDINAL, 32, PropModeReplace, (unsigned char *)val, 4);
   
   XChangeProperty(w->dpy, w->root, w->atoms[_NET_DESKTOP_GEOMETRY],
 		  XA_CARDINAL, 32, PropModeReplace, (unsigned char *)val, 4);
+
 }
 
 void 
@@ -415,16 +440,19 @@ ewmh_state_set(Client *c)
   /* We need to set ewmh state as some apps need to know when they 
    * return from fullscreen
    */
-
   if (c->flags & CLIENT_FULLSCREEN_FLAG)
     atom_states[n_atom_states++] = w->atoms[WINDOW_STATE_FULLSCREEN];
 
   if  (c->flags & CLIENT_IS_MODAL_FLAG)
     atom_states[n_atom_states++] = w->atoms[WINDOW_STATE_MODAL];
 
-  XChangeProperty(w->dpy, c->window, w->atoms[WINDOW_STATE],
-		  XA_ATOM, 32, PropModeReplace, 
-		  (unsigned char *)atom_states, n_atom_states);
+  /* XXX Should avoid trashing states here we dont handle. 
+   *     Otherwise things like skip taskbar state get trashed. 
+  */
+  if (n_atom_states)
+    XChangeProperty(w->dpy, c->window, w->atoms[WINDOW_STATE],
+		    XA_ATOM, 32, PropModeReplace, 
+		    (unsigned char *)atom_states, n_atom_states);
 }
 
 Bool 
@@ -484,6 +512,28 @@ ewmh_set_allowed_actions(Wm *w, Client *c)
 
 void 
 ewmh_set_active(Wm *w)
+{
+
+  static unsigned long last_active; 
+  unsigned long        val[1] = { 0 };
+
+  if (w->focused_client != NULL)
+    val[0] = w->focused_client->window;
+
+  if (last_active == val[0]) 	/* avoid the roundtrip if pos */
+    return;
+
+  last_active = val[0]; 
+
+  dbg("%s() setting %li as active\n", __func__, val[0] );
+
+  XChangeProperty(w->dpy, w->root, w->atoms[_NET_ACTIVE_WINDOW] ,
+		  XA_WINDOW, 32, PropModeReplace,
+		  (unsigned char *)val, 1);
+}
+
+void 
+ewmh_set_current_app_window(Wm *w)
 {
   Client              *c;
   static unsigned long last_active; 
@@ -497,9 +547,7 @@ ewmh_set_active(Wm *w)
 
   last_active = val[0]; 
 
-  dbg("%s() setting %li as active\n", __func__, val[0] );
-
-  XChangeProperty(w->dpy, w->root, w->atoms[_NET_ACTIVE_WINDOW] ,
+  XChangeProperty(w->dpy, w->root, w->atoms[_MB_CURRENT_APP_WINDOW] ,
 		  XA_WINDOW, 32, PropModeReplace,
 		  (unsigned char *)val, 1);
 }
