@@ -84,14 +84,22 @@ main_client_manage_toolbars_for_fullscreen(Client *c, Bool main_client_showing)
 
   stack_enumerate(w, p)
     {
-      if (p->type == MBCLIENT_TYPE_DIALOG 
-	  && p->flags & CLIENT_TB_ALT_TRANS_FOR_APP
-	  && p->trans == c)
+      if (p->type == MBCLIENT_TYPE_DIALOG)
 	{
-	  toolbar_client_configure(p);
-	  toolbar_client_move_resize(p);
+	  if (p->flags & CLIENT_TB_ALT_TRANS_FOR_APP 
+	      && p->trans == c)
+	    {
+	      toolbar_client_configure(p);
+	      toolbar_client_move_resize(p);
 
-	  return p->height;
+	      return p->height;
+	    }
+	  else if (p->flags & CLIENT_TB_ALT_TRANS_FOR_DIALOG)
+	    {
+	      toolbar_client_configure(p);
+	      toolbar_client_move_resize(p);
+	      return 0;
+	    }
 	}
     }
 
@@ -190,8 +198,14 @@ main_client_configure(Client *c)
 	 - wm_get_offsets_size(c->wm, EAST,  NULL, False)
 	 - wm_get_offsets_size(c->wm, WEST,  NULL, False);
 
+#ifdef USE_ALT_INPUT_WIN
+       c->height = c->wm->dpy_height - c->y - h - offset_south - main_client_manage_toolbars_for_fullscreen(c, False);
+#else
        c->height = c->wm->dpy_height - c->y - h - offset_south;
        main_client_manage_toolbars_for_fullscreen(c, False);
+#endif
+
+
      }
 
    dbg("%s() configured as %i*%i+%i+%i, frame size is %i\n", 
@@ -582,7 +596,9 @@ void main_client_button_press(Client *c, XButtonEvent *e)
 	    * For now its just the custom type. May make sense to
             * Add more.
 	   */
-	   if (button_item && button_item->id == BUTTON_ACTION_CUSTOM)
+	   if ((button_item && button_item->id == BUTTON_ACTION_CUSTOM)
+	       || (button_item && button_item->id == BUTTON_ACTION_MIN)
+	       || (button_item && button_item->id == BUTTON_ACTION_CLOSE))
 	     return;
 	 }
      }
