@@ -149,7 +149,7 @@ void theme_frame_icon_paint(MBTheme *t, Client *c,
   if (img->width != 16 || img->height != 16) 
     {
       MBPixbufImage *tmp_img;
-      tmp_img = mb_pixbuf_img_scale_down(t->wm->pb, img, 16, 16); 
+      tmp_img = mb_pixbuf_img_scale(t->wm->pb, img, 16, 16); 
       mb_pixbuf_img_free(t->wm->pb, img);
       img = tmp_img;
     }
@@ -2443,10 +2443,15 @@ mbtheme_init (Wm   *w,
   Nlist *n;
 
   XMLParser *parser = xml_parser_new();
-  
+
   char theme_filename[255] = DEFAULTTHEME;
   char *theme_path = NULL;
-  char orig_wd[255];
+
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 256
+#endif
+
+  char orig_wd[MAXPATHLEN];
   
   if (theme_name != NULL) { 
     if (theme_name[0] == '/')
@@ -2472,7 +2477,7 @@ mbtheme_init (Wm   *w,
       strncpy(theme_filename, DEFAULTTHEME, 255);
     }
   
-  if (getcwd(orig_wd, 255) == (char *)NULL)
+  if (getcwd(orig_wd, MAXPATHLEN) == (char *)NULL)
     {
       fprintf(stderr, "matchbox: cant get current directory\n");
       exit(1);
