@@ -18,6 +18,8 @@
  *
  */
 
+#define _GNU_SOURCE
+
 #ifndef _STRUCTS_H_
 #define _STRUCTS_H_
 
@@ -73,6 +75,10 @@
 #include <X11/extensions/sync.h>
 #endif
 
+#if USE_SM
+#include <X11/SM/SMlib.h>
+#endif
+
 #define GENERIC_ICON        PKGDATADIR "/mbnoapp.xpm"
 
 #ifdef MB_HAVE_PNG
@@ -129,14 +135,16 @@
 #define SHADOW_OFFSET_X	(-SHADOW_RADIUS)
 #define SHADOW_OFFSET_Y	(-SHADOW_RADIUS)
 
-/* Defines for MB_COMMAND values */
+/* Defines for MB_COMMAND values, for matchbox-remote */
 
-#define MB_CMD_SET_THEME 1
-#define MB_CMD_EXIT      2
-#define MB_CMD_DESKTOP   3
-#define MB_CMD_NEXT      4
-#define MB_CMD_PREV      5
-#define MB_CMD_MISC      7 	/* spare, used for debugging */
+#define MB_CMD_SET_THEME   1
+#define MB_CMD_EXIT        2
+#define MB_CMD_DESKTOP     3
+#define MB_CMD_NEXT        4
+#define MB_CMD_PREV        5
+#define MB_CMD_MISC        7 	/* spare, used for debugging */
+#define MB_CMD_COMPOSITE   8
+#define MB_CMB_KEYS_RELOAD 9
 
 /* Atoms, if you change these check ewmh_init() first */
 
@@ -206,6 +214,7 @@ enum {
   WM_TRANSIENT_FOR,
   _NET_WM_SYNC_REQUEST_COUNTER,
   _NET_WM_SYNC_REQUEST,
+  _MB_CURRENT_APP_WINDOW,
   ATOM_COUNT
 
 } MBAtomEnum;
@@ -283,6 +292,7 @@ typedef struct _mb_client_button
 #define CLIENT_IS_TRANSIENT_FOR_ROOT (1<<25)
 #define CLIENT_HAS_URGENCY_FLAG (1<<26)
 #define CLIENT_HAS_ABOVE_STATE (1<<27)
+#define CLIENT_IS_MENU_DIALOG ( 1<<30)
 
 #ifdef USE_ALT_INPUT_WIN
 #define CLIENT_TB_ALT_TRANS_FOR_DIALOG (1<<28)
@@ -438,6 +448,7 @@ typedef struct _kbdconfig_entry
   int                      idata;
   struct _kbdconfig_entry *next_entry;
 
+
 } MBConfigKbdEntry;
 
 typedef struct _kbdconfig
@@ -486,6 +497,7 @@ typedef struct _wm_config
   MBConfigKbd *kb;
 
   char        *force_dialogs;
+  char        *sm_client_id;
 } Wm_config;
 
 
@@ -636,6 +648,11 @@ typedef struct _wm
   int               sync_error_base;
 #endif
 
+#if USE_SM
+  int               sm_ice_fd;      
+  IceConn           ice_conn;
+#endif
+
 #ifdef STANDALONE
   Bool             have_toolbar_panel;
   int              toolbar_panel_x;
