@@ -68,6 +68,10 @@ wm_new(int argc, char **argv)
    w->n_active_ping_clients    = 0;
    w->next_click_is_not_double = True;
 
+#ifdef USE_SM
+   sm_connect(w); 	/* previous_cliend_id */
+#endif
+
    sattr.event_mask =  SubstructureRedirectMask
                        |SubstructureNotifyMask
                        |StructureNotifyMask
@@ -172,7 +176,7 @@ wm_new(int argc, char **argv)
 void
 wm_usage(char *progname)
 {
-   printf("usage: %s %s [options ...]\n", progname, VERSION);
+   printf("usage: %s [options ...]\n", progname);
    printf("\t-display          <string> \n");
    printf("\t-theme            <string> \n");
    printf("\t-use_titlebar     <yes|no>\n");
@@ -183,6 +187,10 @@ wm_usage(char *progname)
    printf("\t-use_dialog_mode  <free|static|const-horiz>\n");
    printf("\t-use_desktop_mode <decorated|plain>\n");
    printf("\t-force_dialogs    <comma seperated list of window titles>\n");
+#ifdef USE_SM
+   printf("\t--sm-client-id    <session id>\n");
+#endif
+
    /*
    printf("\t-ping_handler     <string>\n");
    */
@@ -190,7 +198,7 @@ wm_usage(char *progname)
    printf("\t-titlebar_panel   <x11 geometry>\n");
 #endif
    printf("\n");
-   printf("Compile time options:\n");
+   printf("Compile time features;\n");
 #ifdef DEBUG
    printf("\tdebug build                      yes\n");
 #else
@@ -269,8 +277,16 @@ wm_usage(char *progname)
    printf("\tXRM support                      no\n");
 #endif
 
-   printf("\nVisit http://matchbox.handhelds.org for more info.\n");
-   printf("(c) 2004 OpenedHand Ltd\n");
+#ifdef USE_SM
+   printf("\tSession support                  yes\n");
+#else
+   printf("\tSession support                  no\n");
+#endif   
+
+   printf("\nmatchbox-window-manager %s \nCopyright (C) 2005 Matthew Allum, OpenedHand Ltd.\n", VERSION);
+   printf("This is free software; see the source for copying conditions.\n");
+   printf("Visit http://projects.o-hand.com/matchbox for more info.\n");
+
    exit(0);
 }
 
@@ -285,6 +301,7 @@ wm_load_config (Wm   *w,
    /* Configure these now, and let the arguments override them */
 
    w->config = malloc(sizeof(Wm_config));
+   memset(w->config, 0, sizeof(Wm_config));
 
    /* config defaults */
    w->config->use_title        = True;
@@ -400,13 +417,23 @@ wm_load_config (Wm   *w,
 	  w->config->force_dialogs = argv[i];
 	  continue;
 	}
+
+#ifdef USE_SM
+   if (!strcmp ("--sm-client-id", argv[i]))
+     {
+       if (++i>=*argc) wm_usage (argv[0]);
+       w->config->sm_client_id = strdup(argv[i]);
+       continue;
+     }
+#endif
+
       wm_usage (argv[0]);
    }
 
    if ((w->dpy = XOpenDisplay(w->config->display_name)) == NULL) 
      {
        fprintf(stderr, 
-	       "matchbox: can't open display! check your DISPLAY variable.\n");
+	       "matchbox-wm: can't open display! check your DISPLAY variable.\n");
        exit(1);
      }
 }
@@ -422,7 +449,7 @@ wm_load_config (Wm   *w,
    char              *type;
    XrmValue          value;
    
-   static int opTableEntries = 9;
+   static int opTableEntries = 10;
    static XrmOptionDescRec opTable[] = {
       {"-theme",       ".theme",           XrmoptionSepArg, (XPointer) NULL},
       {"-use_titlebar",".titlebar",        XrmoptionSepArg, (XPointer) NULL},
@@ -433,6 +460,7 @@ wm_load_config (Wm   *w,
       {"-use_desktop_mode",".desktop",     XrmoptionSepArg, (XPointer) NULL},
       {"-titlebar_panel",  ".titlebarpanel", XrmoptionSepArg, (XPointer) NULL},
       {"-force_dialogs",  ".forcedialogs", XrmoptionSepArg, (XPointer) NULL},
+      {"--sm-client-id",  ".session",      XrmoptionSepArg, (XPointer) NULL},
    };
 
    XrmInitialize();
@@ -444,6 +472,7 @@ wm_load_config (Wm   *w,
    XrmCombineDatabase(cmdlnDB, &rDB, True);
 
    w->config = malloc(sizeof(Wm_config));
+   memset(w->config, 0, sizeof(Wm_config));
 
    /* config defaults */
    w->config->use_title        = True;
@@ -600,6 +629,19 @@ wm_load_config (Wm   *w,
      free(geom);
    }
 #endif
+
+#ifdef USE_SM
+   if (XrmGetResource(rDB, "matchbox.session",
+		      "Matchbox.session",
+		      &type, &value) == True)
+     {
+       w->config->sm_client_id = (char *)malloc(sizeof(char)*(value.size+1));
+       strncpy(w->config->sm_client_id, value.addr, (int) value.size);
+       w->config->sm_client_id[value.size] = '\0';
+       dbg("%s() got sm_client_id: %s ", __func__, w->config->sm_client_id);
+     }
+#endif   
+
 }
 #endif /* NO_XRM */
 
@@ -655,34 +697,55 @@ wm_find_client(Wm *w, Window win, int mode)
 
 /* Grab an X Event - block but With a timeout */
 static Bool
-get_xevent_timed(Display        *dpy, 
+get_xevent_timed(Wm             *w,
 		 XEvent         *event_return, 
 		 struct timeval *tv)
 {
   if (tv->tv_usec == 0 && tv->tv_sec == 0)
     {
-      XNextEvent(dpy, event_return);
+#ifndef USE_SM
+      XNextEvent(w->dpy, event_return);
       return True;
+#else
+      tv->tv_sec = 5;
+#endif
     }
 
-  XFlush(dpy);
+  XFlush(w->dpy);
 
-  if (XPending(dpy) == 0) 
+  if (XPending(w->dpy) == 0) 
     {
-      int fd = ConnectionNumber(dpy);
+      int fd = ConnectionNumber(w->dpy);
       fd_set readset;
       FD_ZERO(&readset);
       FD_SET(fd, &readset);
 
+#ifdef USE_SM
+      if (w->sm_ice_fd != -1)
+	{
+	  FD_SET(w->sm_ice_fd, &readset);
+	  if (w->sm_ice_fd > fd) 
+	    fd = w->sm_ice_fd; 	/* for +1 below */
+	}
+#endif
       if (select(fd+1, &readset, NULL, NULL, tv) == 0) 
-	return False;
-      else {
-	XNextEvent(dpy, event_return);
-	return True;
+	{
+	  return False;
+	}
+      else 
+	{
+#ifdef USE_SM
+	  if ( w->sm_ice_fd != -1 && FD_ISSET(w->sm_ice_fd, &readset) )
+	    {
+	      sm_process_event(w);
+	      return False;
+	    }
+#endif
+	  XNextEvent(w->dpy, event_return);
+	  return True;
       }
-
     } else {
-      XNextEvent(dpy, event_return);
+      XNextEvent(w->dpy, event_return);
       return True;
     }
 }
@@ -715,6 +778,9 @@ wm_handle_map_notify(Wm *w, Window win)
 
   if (misc_untrap_xerrors()) return; /* safety on */
 
+  if (attr.class == InputOnly)
+    return;
+
   if (attr.override_redirect)
     {
       dbg("%s() making new overide redirect window\n", __func__);
@@ -770,11 +836,6 @@ wm_event_loop(Wm* w)
 	tvt.tv_sec = 1;
 #endif      
 
-#if MSG_Q
-      if (w->msg_win_queue_head)
-	tvt.tv_sec = 1;
-#endif
-
 #ifdef USE_GCONF
       if (w->gconf_client != NULL)
 	tvt.tv_sec = 1;
@@ -785,7 +846,7 @@ wm_event_loop(Wm* w)
 	tvt.tv_sec = 1;
 #endif
 
-      if (get_xevent_timed(w->dpy, &ev, &tvt))
+      if (get_xevent_timed(w, &ev, &tvt))
 	{
 
 	  switch (ev.type) 
@@ -859,13 +920,6 @@ wm_event_loop(Wm* w)
 	  g_main_context_iteration (w->gconf_context, FALSE);
 #endif
 
-#if MSG_Q
-	if (w->msg_win_queue_head)
-	  {
-	    wm_msg_win_queue_process (w);
-	  }
-#endif
-
 
 #ifndef NO_PING
 	/* check for hung apps every two seconds - they dont last long.. */
@@ -1016,7 +1070,7 @@ wm_handle_keypress(Wm *w, XKeyEvent *e)
 			  p = wm_find_client(w, current_cycle->xid, WINDOW);
 			  if (p)
 			    {
-			      p->show(p);
+			      wm_activate_client(p);
 			      found = True;
 			    }
 			}
@@ -1215,6 +1269,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
    int             req_w      = e->width;
    int             req_h      = e->height;
 
+   Bool            want_activate = False;
+
    /* TODO: Double check ICCCM on this code. 
     *       see http://tronche.com/gui/x/icccm/sec-4.html#s-4.1.5
     *       Only send synthetic configure notify on no change.
@@ -1262,23 +1318,30 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
        /* e->above  is sibling 
         * e->detail is stack_mode 
 	*/
-#ifdef DEBUG
+
        Client *sibling = wm_find_client(w, e->window, WINDOW);
 
-       dbg("%s() value_mask & (CWSibling|CWStackMode) sibling is %s\n",
-	   __func__, sibling ? sibling->name : "unkown" );
+       dbg("%s() value_mask & (CWSibling|CWStackMode) sibling is %s (%li vs %li\n",
+	   __func__, sibling ? sibling->name : "unkown", e->window, c->window );
 
        if (sibling)
 	 {
 	   switch (e->detail)
 	     {
 	     case Above:
+
 	       dbg("%s() (CWSibling|CWStackMode) above %s\n",
 		   __func__, sibling->name);
+
+	       /* Dialog 'above its self, call activate to raise it */
+	       if (sibling == c && c->type == MBCLIENT_TYPE_DIALOG)
+		   want_activate = True;
 	       break;
 	     case Below:
 	       dbg("%s() (CWSibling|CWStackMode) below %s\n",
 		   __func__, sibling->name);
+	       /* Unsupported currently clear flags */
+	       value_mask &= ~(CWSibling|CWStackMode);
 	       break;
 	     case TopIf:    	/* What do these mean ? */
 	     case BottomIf: 
@@ -1286,13 +1349,14 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	     default:
 	       dbg("%s() (CWSibling|CWStackMode) uh? %s\n",
 		   __func__, sibling->name);
+	       /* Unsupported currently clear flags */
+	       value_mask &= ~(CWSibling|CWStackMode);
 	       break;
 
 	     }
 	 }
-#endif       
-       /* Just clear the flags now to be safe */
-       value_mask &= ~(CWSibling|CWStackMode);
+       else /* Just clear the flags now to be safe */
+	 value_mask &= ~(CWSibling|CWStackMode);
      }
 
    if (c->type == MBCLIENT_TYPE_PANEL) 	/* Panels can move */
@@ -1357,7 +1421,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
        /* If any changes, now make them fit it into avaialable area. */
 
        if (req_x != c->x || req_y != c->y 
-	   || req_w != c->width || req_h != c->height)
+	   || req_w != c->width || req_h != c->height
+	   || want_activate)
 	 {
 	   Bool want_fake_configure = False;
 	   
@@ -1383,6 +1448,13 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	    */
 	   if (want_fake_configure)
 	     client_deliver_config(c);
+
+	   /* Client wants to be raised to top of stack */
+	   if (want_activate)
+	     {
+	       dbg("%s(): calling activate client\n", __func__);
+	       wm_activate_client(c);
+	     }
 	   
 	   dialog_client_move_resize(c);
 	   dialog_client_redraw(c, False);
@@ -1526,12 +1598,27 @@ wm_handle_client_message(Wm *w, XClientMessageEvent *e)
 	   wm_toggle_desktop(w);
 	   break;
 	 case MB_CMD_MISC:  /* This is used for random testing stuff */
+	   /* comp_engine_deinit(w); */
 #ifdef DEBUG
 	   /* comp_engine_time(w); Not used atm XXX DO_TIMINGS */
 	   dbg("*** Toggling composite visual debugging ***\n");
 	   w->flags ^= DEBUG_COMPOSITE_VISIBLE_FLAG;
 #endif
 	   break;
+#ifndef NO_KBD
+	 case MB_CMB_KEYS_RELOAD:
+	   keys_reinit(w); 
+	   break;
+#endif
+
+#ifdef USE_COMPOSITE
+	 case MB_CMD_COMPOSITE:
+	   if (w->comp_engine_disabled)
+	     comp_engine_reinit(w);
+	   else
+	     comp_engine_deinit(w);
+	   break;
+#endif
 	 }
        return;
      }
@@ -1609,12 +1696,6 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
       c->trans = NULL;
 
     }
-  else if (e->atom == w->atoms[MB_WIN_SUB_NAME])
-    {
-      if (c->subname) XFree(c->subname);
-      c->subname =ewmh_get_utf8_prop(w, c->window, w->atoms[MB_WIN_SUB_NAME]);
-      update_titlebar = True;
-    }
   else if (e->atom == w->atoms[_NET_WM_NAME])
     {
       if (c->name) 
@@ -1731,40 +1812,7 @@ wm_make_new_client(Wm *w, Window win)
 		   dbg("%s() got desktop atom\n", __func__ );
 		   c = desktop_client_new(w, win);
 		   if (c == NULL) goto end;
-		 }
-#if MSG_Q
-	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
-		 {
-		   dbg("%s() got type message atom\n", __func__ );
-		   if (w->msg_win_queue_head == NULL)
-		     {
-		       dbg("%s() queue empty add win to queue\n", __func__ );
-		       wm_msg_win_queue_add(w, win);
-		     }
-		   
-		   if (win == w->msg_win_queue_head->win)
-		     {
-		       dbg("%s() win is queue head, making client\n", 
-			   __func__ );
-		       c = dialog_client_new(w, win, NULL);
-		       if (c == NULL) goto end;
-		       c->flags |= (CLIENT_IS_MESSAGE_DIALOG|CLIENT_HAS_URGENCY_FLAG); 
-		     }
-		   else
-		     {
-		       dbg("%s() win is not queue head adding to queue\n", 
-			   __func__ );
-		       wm_msg_win_queue_add(w, win);
-		   
-		       dbg("%s() returning from add\n", __func__);
-		       XUngrabServer(w->dpy);
-		       if (value) XFree(value);
-		       return NULL;
-		     }
-		   
-		 }
-#endif
-	       
+		 }	       
 	       else if (value[0] == w->atoms[WINDOW_TYPE_SPLASH])
 		 {
 		   dbg("%s() got splash atom\n", __func__ );
@@ -1772,11 +1820,25 @@ wm_make_new_client(Wm *w, Window win)
 		   if (c == NULL) goto end;
 		   c->flags ^= CLIENT_TITLE_HIDDEN_FLAG;
 		 }
-	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG])
+	       else if (value[0] == w->atoms[WINDOW_TYPE_DIALOG]
+			|| value[0] == w->atoms[WINDOW_TYPE_MENU])
 		 {
 		   dbg("%s() got type dialog atom\n", __func__ );
 		   c = dialog_client_new(w, win, NULL);
 		   if (c == NULL) goto end;
+
+		   /* check to see if we're a 'menu' dialog  */
+		   if (value[0] == w->atoms[WINDOW_TYPE_MENU])
+		     {
+		       dbg("%s() Got menu hint\n", __func__);
+		       c->flags |= CLIENT_IS_MENU_DIALOG;
+		     }
+		 }
+	       else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
+		 {
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags |= CLIENT_HAS_URGENCY_FLAG;
 		 }
 	     } 
 	 }
@@ -1900,7 +1962,7 @@ wm_make_new_client(Wm *w, Window win)
 
    /* Let window know were all done */
 
-   ewmh_state_set(c);
+   ewmh_state_set(c); 		/* XXX This is likely not needed */
 
    client_set_state(c, NormalState);
 
@@ -1921,7 +1983,7 @@ wm_remove_client(Wm *w, Client *c)
 
   XGrabServer(w->dpy);
 
-  misc_trap_xerrors(); 	/* below likely to genrate errors */
+  misc_trap_xerrors(); 	/* below very likely to genrate X errors */
 
   /* Set win state to withdrawn and reparent to root. 
    * If below is not done - suttle bugs appear with toolkits ( GTK )
@@ -1930,11 +1992,13 @@ wm_remove_client(Wm *w, Client *c)
   XReparentWindow(w->dpy, c->window, w->root, 0, 0); 
   XRemoveFromSaveSet(w->dpy, c->window);
 
+  /* sync here so any (likely) lingering X errors are trapped */
+  XSync(w->dpy, False);
+
   misc_untrap_xerrors();
 
   c->destroy(c);
 
-  XSync(w->dpy, False);
   XUngrabServer(w->dpy);
 }
 
@@ -2072,10 +2136,20 @@ wm_update_layout(Wm         *w,
 		 p->height += change_amount;
 		 p->y      -= change_amount;
 		 p->move_resize(p);
-		 theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
-		 theme_pixmap_cache_clear_all(w->mbtheme);
+		 /* XXX shouldn't be any need to redraw here as 
+                  * width won't have changed so no need to repaint 
+                  * entire toolbar. 
+                  *
+                  * It *could* break on if the side decoration themes
+                  * are somehow a function of height. No themes I know
+                  * of do this and we avoid quite a lot of cpu avoiding
+                  * this.
+                  *
+		  * theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
+		  * theme_pixmap_cache_clear_all(w->mbtheme);
+		  * main_client_redraw(p, False);
+		 */
 		 client_deliver_config(p);
-		 main_client_redraw(p, False); /* force title redraw */
 		 break;
 	       case MBCLIENT_TYPE_PANEL :
 		 if (p->flags & CLIENT_DOCK_NORTH
@@ -2093,6 +2167,8 @@ wm_update_layout(Wm         *w,
        }
      else
        {
+	 /* toolbar changes here */
+
 	 dbg("%s(): restack NORMAL comparing %i <= %i for %s\n",
 	     __func__, p->y, client_changed->y, p->name);
 	 if ( (p->y <= client_changed->y) 
@@ -2105,10 +2181,14 @@ wm_update_layout(Wm         *w,
 	       case MBCLIENT_TYPE_APP :
 		 p->height += change_amount;
 		 p->move_resize(p);
-		 theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
-		 theme_pixmap_cache_clear_all(w->mbtheme);
+		 /*
+		  * See above as to why this is commented out.
+                  *
+                  * theme_img_cache_clear( w->mbtheme, FRAME_MAIN );
+		  * theme_pixmap_cache_clear_all(w->mbtheme);
+		  * main_client_redraw(p, False);
+                  */
 		 client_deliver_config(p);
-		 main_client_redraw(p, False); /* force title redraw */
 		 break;
 	       case MBCLIENT_TYPE_TOOLBAR :
 		 p->y += change_amount;
@@ -2169,6 +2249,38 @@ wm_update_layout(Wm         *w,
  ewmh_update_rects(w);
 }
 
+
+static void
+wm_stack_dialogs_for_transient(Wm *w, Client *trans)
+{
+  /* Correctly stack dialogs in respect to a transient ( app/desktop ) 
+   * parent. Utility call for activate_client()
+  */
+  if (trans == NULL)
+    return;
+
+  /* Move transient dialogs to top */
+
+  stack_move_transients_to_top(w, trans, 0);
+
+  /* Above dialogs go above transient for root  */
+
+  stack_move_transients_to_top(w, trans, CLIENT_HAS_ABOVE_STATE 
+			       /* CLIENT_HAS_URGENCY_FLAG */);
+
+  /* Move transient for root dialogs to very top */
+  
+  stack_move_transients_to_top(w, NULL, 0);
+  
+  /* Move transient for root with state above to top  */
+  
+  stack_move_transients_to_top(w, NULL, CLIENT_HAS_ABOVE_STATE );
+  
+  /* Now move transient for root messages to top */
+  
+  stack_move_transients_to_top(w, NULL, CLIENT_HAS_URGENCY_FLAG);
+}
+
 /* wm_activate_client() is called to 'activate', eg raise or show
  * a client stack wise.
  */
@@ -2232,26 +2344,7 @@ wm_activate_client(Client *c)
 
       stack_move_type_above_client(w, MBCLIENT_TYPE_TOOLBAR, c);
 
-      /* Move transient dialogs to top */
-
-      stack_move_transients_to_top(w, c, 0);
-
-      /* Above dialogs go above transient for root  */
-
-      stack_move_transients_to_top(w, c, CLIENT_HAS_ABOVE_STATE 
-				   /* CLIENT_HAS_URGENCY_FLAG */);
-
-      /* Move transient for root dialogs to very top */
-
-      stack_move_transients_to_top(w, NULL, 0);
-
-      /* Move transient for root with state above to top  */
-
-      stack_move_transients_to_top(w, NULL, CLIENT_HAS_ABOVE_STATE );
-
-      /* Now move transient for root messages to top */
-
-      stack_move_transients_to_top(w, NULL, CLIENT_HAS_URGENCY_FLAG);
+      wm_stack_dialogs_for_transient(w, c);
 
       /* Deal with desktop flag etc */
       if (c->type != MBCLIENT_TYPE_DESKTOP
@@ -2277,10 +2370,8 @@ wm_activate_client(Client *c)
 	    }
 	}
 
-      /* We only set active for main clients - this could be wrong, 
-         what about transient for root 'apps' - maybe it should just
-         follow focused client ? */
-      ewmh_set_active(w);
+      /* Set active main client, set focus handles active win hint */
+      ewmh_set_current_app_window(w);
 
     }
   else if (c->type == MBCLIENT_TYPE_DIALOG)
@@ -2288,11 +2379,10 @@ wm_activate_client(Client *c)
       /* A Little insurance - on mapping, a dialog can end up below 
        * panels and toolbars. May be a cleaner way than this.
        */
+      Client *top_main_client = NULL;
 
       if (!(w->flags & DESKTOP_RAISED_FLAG))
 	{
-	  Client *top_main_client = NULL;
-
 	  if ((top_main_client = wm_get_visible_main_client(w)) != NULL)
 	    {
 
@@ -2310,6 +2400,14 @@ wm_activate_client(Client *c)
 	    stack_move_type_above_client(w, MBCLIENT_TYPE_TOOLBAR
 					 |MBCLIENT_TYPE_PANEL, NULL);
 	}
+      else
+	{
+	  /* Desktop is raised. dialogs ( trans for desktop ) can end up 
+           * below panels for some reason on initial map so we take  
+           * below sledgehammer approach which seems to fix things. 
+	  */
+	  wm_stack_dialogs_for_transient(w, wm_get_visible_main_client(w));
+	}
     }
 
 
@@ -2455,7 +2553,7 @@ wm_set_cursor_visibility(Wm *w, Bool visible)
   if (visible)
     {
       w->config->no_cursor = False;
-      w->curs = XCreateFontCursor(w->dpy, XC_right_ptr);
+      w->curs = XCreateFontCursor(w->dpy, XC_left_ptr);
     }
   else
     {
@@ -2589,7 +2687,7 @@ wm_sn_exec(Wm *w, char* name, char* bin_name, char *desc)
       break;
     case 0:
       sn_launcher_context_setup_child_process (context);
-      execlp(bin_name, bin_name, NULL);
+      execlp(bin_name, bin_name, (char *)NULL);
       fprintf (stderr, "Failed to exec %s \n", bin_name);
       _exit (1);
       break;
@@ -3011,110 +3109,4 @@ gconf_key_changed_callback (GConfClient *client,
 
 #endif
 
-#if MSG_Q
-
-void
-wm_msg_win_queue_add(Wm *w, Window win)
-{
-  MsgWinQueue *tmp = w->msg_win_queue_head;
-  int timeout;
-
-  Atom type;
-  int format;
-  long bytes_after;
-  int *data = NULL;
-  long n_items;
-  int result;
-
-  result =  XGetWindowProperty (w->dpy, win, 
-				w->atoms[WINDOW_TYPE_MESSAGE_TIMEOUT],
-				0, 1L,
-				False, XA_CARDINAL,
-				&type, &format, &n_items,
-				&bytes_after, (unsigned char **)&data);
-
-  if (result != Success || data == NULL)
-    {
-      timeout = -1; 		/* No timeout */
-    }
-  else timeout = data[0];
-    
-  if (data) XFree (data);
-
-  dbg("%s() timeout is %i\n", __func__, timeout);
-
-  if (w->msg_win_queue_head == NULL)
-    {
-      dbg("%s() message queue is empty, adding new head\n", __func__);
-      w->msg_win_queue_head = malloc(sizeof(MsgWinQueue));
-      memset(w->msg_win_queue_head, 0, sizeof(MsgWinQueue));
-
-      w->msg_win_queue_head->win = win;
-      w->msg_win_queue_head->timeout = timeout;
-      return;
-    }
-  
-  dbg("%s() queue has items, adding to back \n", __func__);
-
-  while (tmp->next != NULL) tmp = tmp->next;
-
-  tmp->next = malloc(sizeof(MsgWinQueue));
-  memset(tmp->next, 0, sizeof(MsgWinQueue));
-  tmp->next->win = win;
-  tmp->next->timeout = timeout;
-}
-
-
-void
-wm_msg_win_queue_pop(Wm *w)
-{
-  MsgWinQueue *tmp;
-
-  dbg("%s() called\n", __func__);
-
-  if (w->msg_win_queue_head == NULL) return;
-
-  tmp = w->msg_win_queue_head;
-
-  w->msg_win_queue_head = w->msg_win_queue_head->next;
-
-  free(tmp);
-
-  if (w->msg_win_queue_head)
-    {
-      XWindowAttributes doh_attr;
-      if (XGetWindowAttributes(w->dpy, w->msg_win_queue_head->win, &doh_attr))
-	{
-	  wm_make_new_client(w, w->msg_win_queue_head->win);
-	}
-      else /* Eek window no longer exists  */
-	{
-	  wm_msg_win_queue_pop(w); /* so pop again */
-	}
-    }
-}
-
-
-void
-wm_msg_win_queue_process(Wm *w)
-{
-  Client      *client_msg = NULL;
-  if (w->msg_win_queue_head == NULL) return;
-
-  /*
-  dbg("%s() queue head has timeout %i\n", 
-      __func__, w->msg_win_queue_head->timeout );
-  */
-
-  if (w->msg_win_queue_head->timeout > 0) w->msg_win_queue_head->timeout--;
-
-  if (w->msg_win_queue_head->timeout == 0) 
-    if ((client_msg = wm_find_client(w, w->msg_win_queue_head->win, 
-				     WINDOW)) != NULL)
-      {
-	wm_remove_client(w, client_msg);
-      }
-}
-
-#endif
 
